#!/bin/bash

# v0.9 20141202: remove 'update sources.list'
#  copy sources.list to /etc/apt/sources.list DIRECTLY or 'by rc.local'
#  the result is 'show login timeout when entering kernel'
# v0.8 20141128: use previous do_partition()
# v0.7 20141127: 
#  1. modify do_partition(), do_format() for stability
#  2. update /etc/apt/sources.list for i2c-tools
# v0.6 20141125: copy_modules()
#  new method for Gstreamer, 
#  add X Window Driver, EMIO-1533 (VNT-9271) WIFI FW, ASIX AX88772B USB Lan
# v0.5 20131127: update /etc/fstab
# v0.4 20131030: must input parameter! if no parameter, show usage()
# v0.3 20131024: (default)support memSize=0 for detecting DRAM size by gpio
# v0.2 20130927: use script parameters to assign variables
# v0.1: add exit value
#
# memSize={0|1g|2g}, default=0
# bootcmd=[_sd], default=nothing
# u-boot format
#  (a) u-boot.vab820${bootcmd}.bin if memSize==0
#  (b) u-boot.vab820-${memSize}${bootcmd}.bin if memSize!=0
#

usage()
{
 echo
 echo "$(basename $0) accepts the following parameters:"
 echo "  -h|--help"
 echo "  -d|--dev <emmc or /dev/sdX>"
# echo "  -m|--mem <0 or 1g or 2g>          default=0"
# echo "  --bootcmd <spi or sd>          boot command for spi or sd, default is spi"
 echo
 echo "Examples:"
 echo
 echo "# install VAB-820 image to eMMC"
 echo "$(basename $0) -d emmc"
 echo "# install VAB-820 image to /dev/sdf SD card"
 echo "$(basename $0) -d /dev/sdf"
 echo
  
 exit 1
}

check_necessary_files()
{
if [ ! -f $ubootFName ] || [ ! -f $kernelFName ] || [ ! -f $filesystemFName ] ; then
 echo -e "please check files:\n $ubootFName\n $kernelFName\n $filesystemFName\n\n"
 exit 1
fi
}

# $1: storageDev
check_dev()
{
 if [ "$devString" == "eMMC" ]; then return; fi
 # for sd card
 mount | grep " / " | grep "$1" > /dev/null
 if [ "$?" == "0" ]; then
  echo -e "ERROR: Forbidden to write to $1\n"
  exit 1
 fi

 local tmpDev=$(sudo fdisk -l | grep $1)
 if [ "$tmpDev" == "" ]; then
  echo -e "Micro SD card cannot be found in $1.\n"
  exit 1
 fi
}

# $1: storageDev = emmc or /dev/sd[X]
# $2: memSize = 0 or 1g or 2g
# $3: bootcmd = sd or spi
check_variables()
{
 if [ "$1" == "emmc" ]; then
  devString="eMMC"
  # uboot
  bClearUbootPparamter=true
  ubootDev=/dev/mtdblock0
  ubootParamDev=/dev/mtdblock0
  # file system
  fsDev=/dev/mmcblk0
  fsPart=${fsDev}p1
  fsLabel=emmc_820
 else
  devString="micro sd"
  # uboot
  bClearUbootPparamter=false
  ubootDev=$1
  ubootParamDev=/dev/mtdblock0
  # file system
  fsDev=$1
# warning: maybe sd card = /dev/mmcblk[N]
  fsPart=${fsDev}1
  fsLabel=sd_820
 fi
 fsMountFolder=/tmp/$fsLabel
 mainFolder=`pwd`
 filesFolder=${mainFolder}
 local memSize=0
 if [ "$2" == "2g" ]; then 
  memSize=2g; 
 elif [ "$2" == "1g" ]; then
  memSize=1g; 
 fi
 local bootcmd=""
 if [ "$3" == "sd" ]; then bootcmd="_sd"; fi
 if [ "$memSize" == "0" ]; then
  ubootFName=${filesFolder}/u-boot.vab820${bootcmd}.bin
 else
  ubootFName=${filesFolder}/u-boot.vab820-${memSize}${bootcmd}.bin
 fi
 kernelFName=${filesFolder}/uImage.vab820
 # for ltib 4.0.0 rootfs
 #filesystemFName=${filesFolder}/ubuntu_oneiric_4.0.0.tgz
 # for ltib 4.1.0 rootfs
 filesystemFName=${filesFolder}/oneiric.tgz

 check_dev $fsDev
 check_necessary_files
}

parse_parameters()
{
 local storageDev=emmc
 local memSize=0
 local bootcmd=spi
 # if user doesn't input any parameter, show usage() to introduce him/her.
 if [ $# -eq 0 ]; then usage; exit 1; fi
 while [ ! -z "$1" ]; do
  case $1 in
   -h|--help)
    usage
    exit 1
    ;;
   -d|--dev)
    storageDev=$2
    shift 2
    ;;
   -m|--mem)
    memSize=$2
    shift 2
    ;;
   -b|--bootcmd)
    bootcmd="$2"
    shift 2
    ;;
   -r3)
# run partition, format and tar file system
    cmdN3=""; cmdN2=""
    shift
    ;;
   -r2)
# run format, tar file system
    cmdN3="echo NO "; cmdN2=""
    shift
    ;;
   -r0) 
# run NOT partition, format and tar file system
    cmdN3="echo NO "; cmdN2="echo NO "
    shift
    ;;
   -r10)
# NOT run do_boot_media
    cmdN4="echo NO "
    shift
    ;;
   *)
    echo "*** Unknown parameter '$1'!"
    usage
    exit 1
    ;;
  esac
 done
echo "dev=$storageDev, mem=$memSize, bootcmd=$bootcmd"
 check_variables $storageDev $memSize $bootcmd
}

do_umount()
{
sudo umount ${fsDev}* &> /dev/null
sleep 3
}

do_partition()
{
echo "partition $devString..."
sudo dd if=/dev/zero of=$fsDev bs=512 count=1
sleep 3

sudo sfdisk $fsDev -uM << EOF
8,, L,
EOF

sudo blockdev --rereadpt $fsDev
sleep 5
}

do_format()
{
echo -e "format $devString...\n"
if [ ! -b ${fsPart} ]; then
 echo "${fsPart} not found"
 exit 1
fi
sudo mkfs -t ext3 -L $fsLabel ${fsPart}
sleep 10
if [ ! -b ${fsPart} ]; then
 echo "formatted: ${fsPart} not found"
 exit 1
fi
#sudo tune2fs -c 0 ${fsPart}
sleep 1
}

do_mount()
{
echo -e "mount $devString...\n"

if [ ! -d $fsMountFolder ]; then
 sudo mkdir $fsMountFolder
fi
if [ ! -b ${fsPart} ]; then
 echo "${fsPart} not found"
 exit 1
fi
sudo mount ${fsPart} $fsMountFolder
sleep 2
echo "check mount..."
mount
}

# need global variable: fsLabel
# call update_fstab() after extracting RootFS(PS: do_tar())
update_fstab()
{
 local dev="/dev/mmcblk1p1"
 if [ "$fsLabel" == "emmc_820" ]; then
  dev="/dev/mmcblk0p1"
 fi
 echo -e "# UNCONFIGURED FSTAB FOR BASE SYSTEM\n\n${dev} / ext4  errors=remount-ro 0 1" > fstab
 chmod 644 fstab
 sudo chown root:root fstab
 # debug pnly
 sudo cp $fsMountFolder/etc/fstab $fsMountFolder/etc/fstab.test
 sudo rm $fsMountFolder/etc/fstab
 sudo mv fstab $fsMountFolder/etc/fstab
}

do_tar()
{
echo -e "tar file system: $filesystemFName to $devString...\n"

if [ $(date +%Y%m%d) -lt 20130101 ]; then
 # set datetime
 sudo hwclock --set --date="06/18/13 10:10"
 sudo hwclock -s
fi

pushd $fsMountFolder
#sudo tar xvfp $filesystemFName
sudo tar --numeric-owner -xzvf $filesystemFName
popd
sync && sync
sleep 5

update_fstab
}

copy_system_files()
{
pushd ${filesFolder}/
sudo cp rc.local $fsMountFolder/etc/
sudo cp usershutdown.pkla $fsMountFolder/etc/polkit-1/localauthority/50-local.d/
popd
sync && sync
sleep 3
}

copy_modules()
{
pushd ${filesFolder}/
sudo tar -mjxvf modules.tar.bz2 -C ${fsMountFolder}/lib/ 
# Gstreamer
# *****Marked by Ezero -- Begin*****
# sudo cp libmfw_gst_v4lsrc.so ${fsMountFolder}/usr/lib/gstreamer-0.10
# sudo cp libmfw_gst_v4lsink.so ${fsMountFolder}/usr/lib/gstreamer-0.10
# *****Mark by Ezero -- End*****
# *****Add by Ezeto -- Begin*****
sudo tar -jxvf 820_gstreamer-0.10.tar.bz2 -C ${fsMountFolder}/usr
# *****Add by Ezeto -- End*****
popd

# X Window Driver
# *****Add by Ezeto -- Begin*****
sudo cp vivante_drv.so ${fsMountFolder}/usr/lib/xorg/modules/drivers
sudo cp libdri.so ${fsMountFolder}/usr/lib/xorg/modules/extensions
# *****Add by Ezeto -- End*****

# EMIO-1533 (VNT-9271) WIFI FW
# *****Add by Ezeto -- Begin*****
sudo cp htc_9271.fw ${fsMountFolder}/lib/firmware
# *****Add by Ezeto -- End*****

# ASIX AX88772B USB Lan
# *****Add by Ezeto -- Begin*****
tar jxvf asix_lan_driver.tar.bz2 -C $fsMountFolder/home/linaro/Desktop
# *****Add by Ezeto -- End*****

sync && sync
sleep 3
}

copy_x11_patch()
{
tar jxvf 820_x11_hw_accel.tar.bz2 -C $fsMountFolder/home/linaro/Desktop
tar jxvf graphics_sample.tar.bz2 -C $fsMountFolder/home/linaro/Desktop
sync && sync
sleep 3
}

copy_can_utils()
{
sudo tar jxvf can_utils.tar.bz2 -C $fsMountFolder/usr/local/sbin
}

do_uboot()
{
 echo "copy uboot..."
 sudo dd if=$ubootFName of=${ubootDev} bs=512 seek=2 skip=2
 sleep 3
}

do_kernel()
{
sudo cp $kernelFName ${fsMountFolder}/boot/
sync && sync
sleep 3
}

do_boot_media()
{
do_umount
$cmdN3 do_partition
$cmdN2 do_format
do_uboot

do_mount

$cmdN2 do_tar
copy_system_files
copy_modules
copy_x11_patch
copy_can_utils
do_kernel
sync && sync
sync && sync
sleep 3

do_umount
}

# main() .... DO NOT add exit [n]
parse_parameters $@
$cmdN4 do_boot_media

